@@ -2,8 +2,7 @@ import subprocess
 import random
 from rich import inspect, print
 from rich.table import Table
-from . import tools
-from .models import Session
+from . import tools, models
 
 
 def execute(command):
@@ -19,8 +18,8 @@ def execute(command):
     return result
 
 
-def stop(session: Session):
-    if session.active:
+def stop(session: models.Session):
+    if session.connected:
         print("Stop session {tmux_id}.".format(tmux_id=session.tmux_id))
         stop_command = "tmux kill-session -t '{tmux_id}'".format(tmux_id=session.tmux_id)
         execute(stop_command)
@@ -46,44 +45,40 @@ def generate_new_port(port_list):
 
 def start(session, reconnect: bool = False):
     settings = tools.settings()
-    if reconnect and session.active:
+    if reconnect and session.connected:
         # print(f"[b]Stopping '{session_name}'[/b]")
         stop(session)
 
-    if not session.active:
+    if not session.connected:
         generate_port = False
 
-        if 'local_port' not in session:
+        if not session.local_port:
             generate_port = True
 
         if session.local_port in settings.ports_active:
-            print(f"[b]Warning[/b]: Local port {session.local_port} already in use, assigning new port")
-            generate_port = True
+            if session.local_port_dynamic:
+                print(f"[b]Warning[/b]: Local port {session.local_port} already in use, assigning new port")
+                generate_port = True
+            else:
+                print(f"[b]Warning[/b]: Local port {session.local_port} already in use")
+                exit()
 
         if generate_port:
             session.local_port = generate_new_port(settings.ports_active)
 
-        session_data = session.dict()
-        print("Start {type} forward on [b]{hostname}[/b] {remote_address}:{remote_port}.".format(**session_data))
-
-        session_data['tmux_id'] = session.tmux_id
-
-        session_data['shell_command'] = 'ping localhost'
-
-        if session_data.get('type') == 'remote':
-            ssh = 'ssh -o ExitOnForwardFailure=yes -R {local_address}:{local_port}:{remote_address}:{remote_port} {hostname}'
-        else:
-            ssh = 'ssh -o ExitOnForwardFailure=yes -L {local_address}:{local_port}:{remote_address}:{remote_port} {hostname}'
-
-        session_data['ssh_command'] = ssh.format(**session_data)
-
-        # start_command = "screen -dmS '{name}' {ssh_command}  -- {shell_command}"
-        start_command = "tmux new-session -d -s '{tmux_id}' '{ssh_command} -- {shell_command}'"
-        # inspect(session_definition)
-        result = execute(start_command.format(**session_data))
+        result = execute(session.command)
         return result
     else:
-        print("Ignoring {type} forward on [b]{hostname}[/b] {remote_address}:{remote_port}.".format(**session.dict()))
+        print("Ignoring {type} forward on [b]{hostname}[/b] {remote_address}:{remote_port}.".format(**session.as_dict()))
+
+
+def refresh_state():
+    update_state()
+    for session in models.Session.index():
+        if not session.active or session.connected:
+            continue
+
+        start(session)
 
 
 def update_state():
@@ -91,13 +86,11 @@ def update_state():
 
     # Reset state
     settings.ports_active = []
-    schema_list = tools.get_schemas_ids()
-    for schema_name in schema_list:
-        schema = tools.get_schema(schema_name)
-        schema.active = False
+    for schema in models.Schema.index():
+        # schema.active = False
 
         for session in schema.sessions:
-            session.active = False
+            session.connected = False
 
     # Update state from tmux sessions
     command = "tmux ls | grep 'pfm_session' | cut -d ':' -f 1"
@@ -114,20 +107,19 @@ def update_state():
         values = session_data.split('|')
 
         if len(values) == 8:
-            filler, schema_id, hostname, remote_host, remote_port, local_address, local_port, forward_type = values
+            filler, schema_name, hostname, remote_host, remote_port, local_address, local_port, forward_type = values
 
-            schema = tools.get_schema(schema_id)
-            schema.active = True
-            session = schema.get_session(hostname, int(remote_port))
-            session.schema_id = schema.id
-            session.active = True
-            session.local_address = local_address
+            schema = models.Schema.find_by_name(schema_name)
+            # schema.active = True
+            session = schema.get_session(forward_type, hostname, int(remote_port))
+            session.connected = True
+            # session.local_address = local_address
             session.local_port = local_port
             settings.ports_active.append(local_port)
 
 
-def filter_session(session: Session, schema: str = None, host: str = None, port: str = None):
-    if schema and schema not in session.schema_id:
+def filter_session(session: models.Session, schema_name: str = None, host: str = None, port: str = None):
+    if schema_name and schema_name not in session.schema.name:
         return True
     if host and host not in session.hostname:
         return True
@@ -138,10 +130,7 @@ def filter_session(session: Session, schema: str = None, host: str = None, port:
 def list_from_active(key, filter_string: str = ''):
     items = []
 
-    schema_list = tools.get_schemas_ids()
-    for schema_name in schema_list:
-        schema = tools.get_schema(schema_name)
-
+    for schema in models.Schema.index():
         for session in schema.sessions:
             if not session.active:
                 continue
@@ -160,14 +149,14 @@ def prepare_table(show_alias: bool = False, show_link: bool = False, show_schema
     table = Table(show_edge=settings.show_table_border)
 
     if show_schema:
-        table.add_column("Schema", style="green", width=20)
+        table.add_column("Schema", style="green", width=30)
 
     table.add_column("Hostname", justify="left", style="yellow", width=30)
     table.add_column("Type", justify="right", style="yellow", width=8)
-    table.add_column("Local host", justify="right", style="white", width=15)
+    table.add_column("Local address", justify="right", style="white", width=15)
     table.add_column("Local port", justify="right", style="white", width=7)
-    table.add_column("Remote host", justify="right", style="yellow", width=15)
-    table.add_column("Remote port", justify="right", style="yellow", width=7)
+    table.add_column("Remote address", justify="right", style="blue", width=15)
+    table.add_column("Remote port", justify="right", style="blue", width=7)
 
     if show_alias:
         table.add_column("Alias", justify="left", style="cyan", width=25)
@@ -181,29 +170,25 @@ def prepare_table(show_alias: bool = False, show_link: bool = False, show_schema
 def show_active_sessions(schema: str = None, host: str = None, port: int = None):
     update_state()
 
-    schema_list = tools.get_schemas_ids()
-    settings = tools.settings()
     table = prepare_table(True, True, True)
     session_count = 0
-    for schema_name in schema_list:
-        schema = tools.get_schema(schema_name)
-
+    for schema in models.Schema.index():
         schema_rows = False
         for session in schema.sessions:
-            if not session.active:
+            if not session.connected:
                 continue
 
             schema_rows = True
 
             row = [
-                schema_name,
+                schema.name,
                 session.hostname,
                 session.type,
                 session.local_address,
                 'auto' if session.local_port == 0 else session.local_port.__str__(),
                 session.remote_address,
                 session.remote_port.__str__(),
-                session.alias,
+                session.label,
                 session.url
             ]
             table.add_row(*row)
@@ -216,14 +201,3 @@ def show_active_sessions(schema: str = None, host: str = None, port: int = None)
     else:
         print("Nothing to see here")
 
-
-def get_session_definition(session):
-    schema = tools.get_schema(session.get('schema'))
-    for session_definition in schema:
-        if session_definition.get('hostname') != session.get('hostname'):
-            continue
-        if session_definition.get('remote_port') != session.get('remote_port'):
-            continue
-
-        return session_definition
-
